/*****************************************
* data.c - Data Manipulation Functions
*
* CVS $Id: data.c,v 1.12 2001/02/14 01:20:31 jroes Exp $
*****************************************/

#include "split.h"

void InitTables()
{
	int i;

        for (i=0; i<MAXSERVER; i++) {
                server_table[i] = NULL;
        }
        map_head = NewServer(me.name, NULL);
        for (i=0; i<MAXSERVER; i++) {
                split_table[i] = NULL;
        }
}

Server* NewServer(char* name, char* uplink)
{
	int i,j;
	Server* up = FindServer(uplink);
	Server* s = malloc(sizeof(Server));
	s->name = name;
	s->uplink = up;
	s->numleaf = 0;
	for (i=0; i<MAXSERVER; i++)
	{
		s->leaflist[i] = NULL;
	}
	if (up)
	{
		up->numleaf++;
		for(j=0; j<MAXSERVER; j++)
		{
			if (!up->leaflist[j]) {
				up->leaflist[j] = s;
				break;
			}
		}
	} else {
		log("WARNING: Unable to find uplink: %s for server name: %s",
			uplink, name);
	}
	for (i=0; i<MAXSERVER; i++)
	{
		if (!server_table[i]) {
			server_table[i] = s;
			break;
		}
	}
	return s;
}

void DelServer(char* name)
{
	int i;
	int j;
	Server* s = NULL;
	if (name) {
		for (j=0; j<MAXSERVER; j++) {
			if (server_table[j]) {
				if (!strcasecmp(name, server_table[j]->name)) {
					s = server_table[j];
					server_table[j] = NULL;
				}
			}
		}
		if (s) {
		    if (s->uplink) {
			s->uplink->numleaf--;

			for (i=0; i<MAXSERVER; i++) {
				if (s->uplink->leaflist[i]) {
					if (!strcasecmp(name, 
					    s->uplink->leaflist[i]->name)) {
						s->uplink->leaflist[i] = NULL;
					}
				}
			}

                    } else {
                        log("No uplink for server: %s", s->name);
		    }

		    free(s);
		} else {
		    log("No server table entry for server name: %s", name);
		}
	}
}

Server* FindServer(char* name)
{
	int i;
	if (name) {
		for (i=0; i<MAXSERVER; i++)
		{
			if (server_table[i]) {
				if (!strcasecmp(server_table[i]->name, name)) {
					return server_table[i];
				}
			}
		}
	}
	log("Unable to FindServer for name: %s", name);
	return NULL;
}

void NewSplit(char * name)
{
	Server* s = malloc(sizeof(Server));
	Server *t = FindSplit(name);
	int i;
	if (t) {
		log("Split entry for server name: %s already exists", name);
		return;
	}
	s->name = name;
	s->ts = time(NULL);
	/* None of the other variables are used here */
	for (i=0; i<MAXSERVER; i++) {
		if (!split_table[i]) {
			split_table[i] = s;
			break;
		}
	}
}

void DelSplit(char * name)
{
	Server* s = NULL;
	int i;
	for (i=0; i<MAXSERVER; i++) {
		if (split_table[i]) {
			if (!strcasecmp(split_table[i]->name, name)) {
				s = split_table[i];
				split_table[i] = NULL;
				break;
			}
		}
	}
	if (s) {
		free(s);
	} else {
		log("Unable to DelSplit server name: %s", name);
	}
}

Server* FindSplit(char* name)
{
        int i;
        if (name) {
                for (i=0; i<MAXSERVER; i++)
                {
                        if (split_table[i]) {
                                if (!strcasecmp(split_table[i]->name, name)) {
                                        return split_table[i];
                                }
                        }
                }
        }
	log("Unable to FindSplit server name: %s", name);
        return NULL;
}
