/**********************************************
* main.c - The main system file for SplitServ.
*          Ties all the other pieces together
*
* CVS $Id: main.c,v 1.17 2001/01/21 02:46:38 wcampbel Exp $
**********************************************/

#include "split.h"
#include "patchlevel.h"

int main(int argc, char *argv[])
{
	int forked = 0;
	FILE* fp;

	sprintf(me.version, "%s(%s) (Compile: %s %s)",
		PATCHLEVEL, SERIALNUM, __DATE__, __TIME__);

	InitDefaults();

	LoadConf();

	InitTables();

	forked=fork();
	if (forked) {
		fp = fopen(PIDFILE, "w");
		fprintf(fp, "%i", forked);
		fclose(fp);
		return 0;
	} 

        notice = time(NULL) + me.delay;
        is_delayed = 1;
        ResetTimers();

	while(1) {
		servsock = Connect();
        	if (servsock <= 0) {
                	log("Unable to connect to %s", me.uplink); 
	        } else {
			Register();
			ListenLoop();
		}
		log("Reconnecting to the server in %d seconds", me.r_time);

		/* Now we get rid of all our uplink's SERVER table */
		FlushServerTree(map_uplink);
		sleep(me.r_time);
		/* Reset the variables used in initialization to prevent the
		** service from spewing large chunks of data out to the 
		** channel
		*/
		notice = time(NULL) + me.delay + me.r_time;
		is_delayed = 1;
	}
	return 0;
}

void FlushServerTree(Server *s)
{
	int i;
	if (s) {
		for (i=0; i<MAXSERVER; i++) {
			if(s->leaflist[i]) {
				FlushServerTree(s->leaflist[i]);
				s->leaflist[i]=NULL;
			}
		}
		/* Damn, extra calls to findserver :( */
		DelServer(s->name);
#if 0
		/* I shouldn't add things to the split table */
		NewSplit(s->name);
#endif
	}
}
