/**************************************************************
* SplitServ main header file
* $Id: split.h,v 1.43 2001/01/18 14:16:07 wcampbel Exp $
**************************************************************/

#ifndef SPLIT_H
#define SPLIT_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/time.h>
#include <ctype.h>
#include <sys/stat.h>
#include "config.h"

#define BUFSIZE 512
#define MAXSERVER 64
#define bzero(x,y)	memset(x, '\0', y);
#define MAXNICK	32
#define MAXUSER	10
#define MAXHOST	63
#define MAXPASS	32
#define MAXDEPTH 15
#define LINE 80
#define TRUE 1
#define FALSE 0

typedef struct server_ Server;

struct server_ {
	char* name;
	Server* uplink;
	int numleaf;
	time_t ts;
	Server* leaflist[MAXSERVER];
};

struct me {
	char uplink[MAXHOST];
	char name[MAXHOST];
	int port;
	char desc[LINE];
	int r_time;
	char pass[MAXPASS];
	char adpass[MAXPASS];
	char version[LINE];
	int privmsg : 1;
	int chanmsg : 1;
	int numserver;
	char join_chan[LINE];
	int delay;
	int enable_text : 1;
	int text_time;
	char text_name[BUFSIZE];
} me;

struct SplitServ {
	char host[MAXHOST];
	char user[MAXUSER];
	char nick[MAXNICK];
} SplitServ;

int servsock;
time_t notice;
int is_delayed;
Server* split_table[MAXSERVER];
Server* server_table[MAXSERVER];
Server* map_head;
Server* map_uplink;

/* conf.c */
void InitDefaults();
void strip(char *);
void LoadConf();

/* sock.c */
void ListenLoop();
int Connect();

/* main.c */
void FlushServerTree(Server *);

/* irc.c */
void Register();
void InitSplitServ();
void Parse(char *);
void PrintSplit(char *);
void PrintMap(char *, Server *, int);
void PrintDump(char *);
char* dash(int);
void Shutdown(char *, char *, char *);
void Restart(char *, char *, char *);
void Update(char *, char *);
void Add(char *, char *, char *);
void Forget(char *, char *, char *);
void PrintHelp(char *, char *);

/* util.c */
void log(char *, ...);
void sts(char *, ...);
void msg_list(char *, const char **);
char *timediff(time_t);
void CTCPClean(char *);
void chk();
void ResetTimers();
void privmsg(char *, char *, ...);
int is_channel(char *);
void dump_map();
void dump_text(FILE*, Server*, int);

/* data.c */
void InitTables();
Server* NewServer(char *, char *);
Server* FindServer(char *);
void DelServer(char *);
void NewSplit(char *);
void DelSplit(char *);
Server* FindSplit(char *);

/* help.c */
extern const char *help_main[];
extern const char *help_pub[];
extern const char *help_map[];
extern const char *help_split[];
extern const char *help_dump[];
extern const char *help_shutdown[];
extern const char *help_restart[];
extern const char *help_forget[];
extern const char *help_add[];

#endif
